//Source file: $Id: FTAActor.java,v 1.15 2001/06/21 14:57:09 veit Exp $

// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

package jfta.actors;

import java.rmi.*;

import jfta.exception.*;
import jfta.ensemble.*;
import jfta.base.*;
import jfta.tagboard.*;
import jfta.system.*;
import jfta.tags.*;
import jfta.http.*;
import jfta.bootstrap.*;

/*
  The actor is a metaclass for user actors.
  It contains the user written method eval(),
  and provides the following convenience services:
  add/removeHost
  add/removeEnsemble
  add/removeTagboard
  addActor
  get/putTag
  getEnsemble/Tagboardtime
  findHost/Ensemble/Tagboard
  add/remove EE/ET sync
*/

/**
 * Class <code>FTAActor</code> is the base class of all actors.
 * To create your own actor, <b>extend</b> your class from <code>FTAActor</code>
 * and <b>implement</b> the interface <code>FTAActorI</code>.
 * @author   Holger Veit, Elmar Degen
 * @version  $Id: FTAActor.java,v 1.15 2001/06/21 14:57:09 veit Exp $
 * @see      jfta.actors.FTAActorI
 * @since    JFTA1.0
 */
public class FTAActor extends Object
    implements FTAActorI,FTAServletI,FTAEvalableI {

    /**
     * constant for logMessage
     */
    public final static int LOG_STD = FTADaemon.LOG_STD;
    public final static int LOG_ERROR = FTADaemon.LOG_ERROR;
    public final static int LOG_WARNING = FTADaemon.LOG_WARNING;
    public final static int LOG_SUCCESS = FTADaemon.LOG_SUCCESS;
    public final static int LOG_USER = FTADaemon.LOG_USER;
    //public final static int KEEPALIVE = FTAEvalableI.KEEPALIVE;
    //public final static int TERMINATE = FTAEvalableI.TERMINATE;
    //public final static int YIELD = FTAEvalableI.YIELD;

    /**
     * constant used for getTagboardTime/getEnsembleTime
     * @see FTAActor#getTagboardTime(String,int)
     * @see FTAActor#getEnsembleTime(String,int)
     */
    public final static int ABSOLUTE = 1;
    public final static int RELATIVE = 2;

    /**
     * constant used for getProfileInfo
     * @see FTAActor#getProfileInfo(int)
     */
    public static final int PROFMIN = 1;
    public static final int PROFMAX = 2;
    public static final int PROFAVG = 3;
    public static final int PROFLAST = 4;

    /**
     * @serial stores the name of the actor
     */
    private String _myname;
    /**
     * @serial stores the reference of the own ensemble
     */
    private transient FTAEnsembleSub _myensemble;
    /**
     * @serial stores the reference to the local system office
     */
    private transient FTASysSub _mysyssub;
    /**
     * @serial a temporary storage for the <code>FTAService</code>
     * helper class
     */
    public final static FTAService _svc = FTAService.Instance();

    /**
     * internal use
     */
    private transient boolean _secondrun;

    /**
     * internal use only, don't touch
     */
    public final boolean isEval() {
	if (!_secondrun) {
	    _secondrun = true;
	    return false;
	}
	return _myensemble.getRunFlag();
    }
  
    /**
     * Returns the name of the actor.
     * @return  the name of the actor as a <code>String</code>
     * @roseuid 38B3C2E8ECB8
     */
    public final String getName() {
	return _myname;
    }

    /**
     * Returns the name of the actor.
     * @return  the name of the actor as a <code>String</code>
     * @roseuid 38B3C2E8ECB8
     */
    public final String toString() {
	return _myname;
    }

    /**
     * Return the real actor type.
     * @return the class type of the actor
     */
    public String getType() {
	return getClass().getName();
    }
    
    /**
     * Return a hash code for the actor. This is used internally
     * in the management of actors in an ensemble.
     * @return a hash code as an <code>int</code> value.
     * @roseuid 38B3C2E8ECB9
     */
    public final int hashCode() {
	return _myname.hashCode();
    }
  
    /**
     * Register a new host for use by the current FTA system.
     * @param hostname  the fully qualified domain name of a host
     * @exception FTACreationException if the creation failed
     * @roseuid 38B3C2E8ECBA
     */
    public final void addHost(String hostname)
	throws FTACreationException {
	_mysyssub.addHost(hostname);
    }
  
    /**
     * Add a new tagboard to the current FTA system.
     * @param tbname  logical name of the tagboard. Must be unique in
     *   this FTA system.
     * @param hostname  fully qualified hostname (has to be registered
     *   with {@link #addHost addHost}). This is obsolete
     * @param flags creation type
     * @exception FTACreationException  the tagboard could not be
     *   created.
     * @deprecated @see #addTagboard(String,int) instead
     */
    public final void addTagboard(String tbname, String hostname,
				     int flags)
	throws FTACreationException {
	addTagboard(tbname,flags);
    }

    /**
     * Add a new tagboard to the current FTA system.
     * @param tbname  logical name of the tagboard. Must be unique in
     *   this FTA system.
     * @param flags creation type
     * @exception FTACreationException  the tagboard could not be
     *   created.
     */
    public final void addTagboard(String tbname,int flags)
	throws FTACreationException {
	_mysyssub.addTagboard(tbname,flags);
    }
  
    /**
     * Add a new ensemble to the FTA system.
     * @param enname  logical name of the ensemble. Must be unique
     *   in this FTA system.
     * @param hostname  fully qualified hostname (has to be registered
     *   with {@link #addHost addHost}). This is obsolete.
     * @param flags creation type
     * @exception FTACreationException  the ensemble could not be
     *   created.
     * @deprecated @see #addEnsemble(String,int)
     */
    public final void addEnsemble(String enname, String hostname,
				     int flags)
	throws FTACreationException {
	addEnsemble(enname,flags);
    }

    /**
     * Add a new ensemble to the FTA system.
     * @param enname  logical name of the ensemble. Must be unique
     *   in this FTA system.
     * @param hostname  fully qualified hostname (has to be registered
     *   with {@link #addHost addHost}). This is obsolete.
     * @param flags creation type
     * @exception FTACreationException  the ensemble could not be
     *   created.
     */
    public final void addEnsemble(String enname, int flags)
	throws FTACreationException {
	_mysyssub.addEnsemble(enname,flags);
    }

    /**
     * Add a new actor to the home ensemble.
     * @param acname  logical name of the actor. Must be unique
     *   in this FTA system.
     * @param actor the actor
     * @exception FTACreationException  the actor couldn't have
     *   been created.
     * @exception FTAUnknownObjectException should not occur in this 
     *   method, is relevant in other flavours of <code>addActor()</code>
     * @see #init <code>init()</code>
     * @see #addActor(String,String,FTAActor) <code>addActor</code>
     * @see #addActor(String,String,String,FTAActor) <code>addActor</code>
     * @see #init <code>init()</code>
     * @roseuid 38B3C2E8ECC4
     */
    public final void addActor(String acname, FTAActor actor)
	throws FTACreationException,FTAUnknownObjectException {
	actor._myname = acname;
	_mysyssub.localAddActor(_myensemble,actor);
    }
  
    /**
     * Add a new actor to the home ensemble.
     * @param acname  logical name of the actor. Must be unique in this FTA
     *   system.
     * @param actype actor type
     * @param args actor args
     * @exception FTACreationException  the actor couldn't have been created.
     * @exception FTAUnknownObjectException should not occur in this 
     *   method, is relevant in other flavours of <code>addActor()</code>
     * @see #init <code>init()</code>
     * @see #addActor(String,String,FTAActor) <code>addActor</code>
     * @see #addActor(String,String,String,FTAActor) <code>addActor</code>
     * @see #init <code>init()</code>
     */
    public final void addActor(String acname,String actype,
				  String[] args)
	throws FTACreationException,FTAUnknownObjectException,
	FTAArgumentException {
	FTAActor actor = _mysyssub.createActor(actype,acname,args);
	_mysyssub.localAddActor(_myensemble,actor);
    }

    /**
     * Add a new actor to an ensemble.
     * @param enname  logical name of the target ensemble.
     *   system.
     * @param acname  logical name of the actor. Must be unique in this FTA
     *   system.
     * @param actor the actor
     * @exception FTACreationException  the actor couldn't have been created.
     * @exception FTAUnknownObjectException the given ensemble does
     *   not exist or is not a distributed object that has a local office
     *   on this host.
     * @see #init <code>init()</code>
     * @see #addActor(String,FTAActor) <code>addActor</code>
     * @see #addActor(String,String,String,FTAActor)
     *   <code>addActor</code>
     * @see #init <code>init()</code>
     * @roseuid 38B3C2E8ECC8
     */
    public final void addActor(String enname, String acname,
				  FTAActor actor)
	throws FTACreationException,FTAUnknownObjectException {
	addActor(null,enname,acname,actor);
    }
  
    /**
     * Add a new actor to an ensemble on a host.
     * @param hostname  fully qualified hostname of the ensemble office where
     *   the actor is supposed to be created.
     * @param acname  logical name of the actor. Must be unique in this FTA
     *   system.
     * @param actor the actor
     * @exception FTACreationException  the actor couldn't have been created.
     * @exception FTAUnknownObjectException the ensemble does
     *  not exist or is not a distributed object that has a local office
     *  on the given host.
     * @see #init <code>init()</code>
     * @see #addActor(String,FTAActor) <code>addActor</code>
     * @see #addActor(String,String,FTAActor) <code>addActor</code>
     * @see #init <code>init()</code>
     * @roseuid 38B3C2E8ECCD
     */
    public final void addActor(String hostname, String enname,
				  String acname, FTAActor actor)
	throws FTACreationException,FTAUnknownObjectException {
	_mysyssub.addActor(hostname,enname,acname,actor);
    }

    /**
     * Submit a new tag to a tagboard.
     * @param tbname logical name of the target tagboard.
     * @param tag an already locally created tag which must be serializable
     * @exception FTAUnknownObjectException the tagboard does not
     *   exist.
     * @exception FTATagException the tag could not be distributed.
     */
    public final void putTag(String tbname, FTATag tag)
	throws FTAUnknownObjectException,FTATagException {
	_mysyssub.putTag(tbname,tag);
    }

    /**
     * Submit a new tag to a tagboard.
     * @param tbname logical name of the target tagboard.
     * @param tag an already locally created tag which must be serializable
     * @exception FTAUnknownObjectException the tagboard does not
     *   exist.
     * @exception FTATagException the tag could not be distributed.
     */
    public final void putTag(String tbname, String name, String[] args)
	throws FTAUnknownObjectException,FTATagException {
	Object tagobj = null;
	try {
	    Class tagclass = Class.forName(name);
	    tagobj = tagclass.newInstance();
	    FTATag tag = (FTATag)tagobj;
	    tag.init(args);
	    _mysyssub.putTag(tbname,tag);
	}
	catch (FTAArgumentException e) {
	    throw new FTATagException("Error in argument");
	}
	catch (FTAUnknownObjectException e) {
	    throw e;
	}
	catch (Exception e) {
	    throw new FTATagException("No such tag: "+name);
	}
    }

    /**
     * Establish an ownership relation between an ensemble and a tagboard
     * @param enname logical name of the ensemble that owns a tagboard
     * @param tbname logical name of the tagboard that is owned
     * @exception FTACreationException the ownership relation couldn't have
     *   been created
     * @exception FTAUnknownObjectException the tagboard or ensemble does not
     *   exist, or is non-distributed, but has no office on the host of the
     *   partner.
     * @roseuid 38B3C2E9D428
     */
    public final void addETSync(String enname, String tbname)
	throws FTACreationException,FTAUnknownObjectException {
	_mysyssub.addETSync(enname,tbname);
    }
  
    /**
     * Remove a host from the current FTA system. All objects that exist
     * on this host will become unavailable afterwards. This function will
     * remove the system office reference for this host, not the
     * <code>FTADaemon</code>.
     * @param hostname fully qualified name of the host to be removed.
     * @exception FTARemovalException the host couldn't be removed
     * @exception FTAUnknownObjectException the host does not exist
     * @roseuid 38B3C2E9E25D
     */
    public final void removeHost(String hostname)
	throws FTARemovalException,FTAUnknownObjectException {
	_mysyssub.removeHost(hostname);
    }
  
    /**
     * Remove an actor from an ensemble. The removal will take place
     * at the end of the ongoing tick.
     * @param enname logical name of the ensemble
     * @param acname logical name of the actor
     * @exception FTARemovalException the actor couldn't be removed
     * @exception FTAUnknownObjectException the ensemble does not exist
     * @roseuid 38B3C2E9E25F
     */
    public final void removeActor(String enname, String acname)
	throws FTARemovalException,FTAUnknownObjectException {
	throw new FTARemovalException("removeActor unimplemented");
    }
  
    /**
     * Remove an ensemble. The removal will take place
     * at the end of the ongoing tick.
     * @param enname logical name of the ensemble
     * @exception FTARemovalException the ensemble couldn't be removed
     * @exception FTAUnknownObjectException the ensemble does not exist
     * @roseuid 38B3C2E9E262
     */
    public final void removeEnsemble(String enname)
	throws FTARemovalException,FTAUnknownObjectException {
	_mysyssub.removeEnsemble(enname);
    }
  
    /**
     * Remove a tagboard. The removal will take place
     * at the end of the ongoing flip.
     * @param tbname logical name of the tagboard
     * @exception FTARemovalException the tagboard couldn't be removed
     * @exception FTAUnknownObjectException the tagboard does not exist
     * @roseuid 38B3C2E9E264
     */
    public final void removeTagboard(String tbname)
	throws FTARemovalException,FTAUnknownObjectException {
	_mysyssub.removeTagboard(tbname);
    }
  
    /**
     * Get a tag iterator to retrieve tags
     * @param tbname logical name of the tagboard
     * @param tagname name of the tag
     * @exception FTAUnknownObjectException the tagboard does not exist
     * @return a usable tag iterator
     * @roseuid 38B3C2E9E266
     */
    public final FTATagIterator getTagIterator(String tbname,
						  String tagname)
	throws FTAUnknownObjectException {
	return _mysyssub.getTagIterator(tbname,tagname);
    }
  
    /**
     * return the current tagboard time
     * @param tbname logical name of the tagboard
     * @param format either return the flip count or the milliseconds
     *   since 1 Jan 1970 of the last time the tagboard flipped
     *   <dl><dt>FTAActor.ABSOLUTE<dd>return milliseconds
     *       <dt>FTAActor.RELATIVE<dd>return flip count</dl>
     * @exception FTAUnknownObjectException the tagboard does not exist
     * @return the flip time (count or absolute time)
     * @roseuid 38B3C2E9E269
     */
    public final long getTagboardTime(String tbname, int format)
	throws FTAUnknownObjectException {
	return _mysyssub.getTagboardTime(tbname,format);
    }
  
    /**
     * return profiling data, i.e. the time required for the tick cycle of
     * this ensemble
     * @param type the information requested
     * @return the profile data
     */
    public final double getProfileInfo(int type) {
	return _myensemble.getProfileInfo(type);
    }

    /**
     * return the current ensemble time
     * @param enname logical name of the ensemble
     * @param format either return the tick count or the milliseconds
     *   since 1 Jan 1970 of the last time the ensemble ticked
     *   <dl><dt>FTAActor.ABSOLUTE<dd>return milliseconds
     *       <dt>FTAActor.RELATIVE<dd>return tick count</dl>
     * @exception FTAUnknownObjectException the ensemble does not exist
     * @return the tick time (count or absolute time)
     * @roseuid 38B3C2E9E26C
     */
    public final long getEnsembleTime(String enname, int format)
	throws FTAUnknownObjectException {
	return _mysyssub.getEnsembleTime(enname,format);
    }
  
    /**
     * get the number of tags on a tagboard that match a certain pattern
     * @param tbname logical name of a tagboard
     * @param tagname typename of tag
     */
    public final int getTagCount(String tbname,String tagname)
	throws FTAUnknownObjectException {
	return _mysyssub.getTagCount(tbname,tagname);
    }
  
    /**
     * Remove an ownership relation between an ensemble and a tagboard
     * @param enname logical name of the ensemble that owns a tagboard
     * @param tbname logical name of the tagboard that is owned
     * @exception FTARemovalException the ownership relation couldn't have
     *   been removed
     * @exception FTAUnknownObjectException the tagboard or ensemble
     *   does not exist, or is non-distributed, but has no office on
     *   the host of the partner, or the tagboard is not owned by
     *   this ensemble
     * @roseuid 38B3C2E9ED39
     */
    public final void removeETSync(String enname, String tbname)
	throws FTARemovalException,FTAUnknownObjectException { 
	_mysyssub.removeETSync(enname,tbname);
    }
  
    /**
     * explicitly request that an (unowned) tagboard flips. Note: this request
     * is collected in the home ensemble, and is transmitted at the end of
     * the cycle. There is no feedback in case the requested tagboard 
     * does not exist, nor is there any guarantee that no others also flip
     * the same board.
     * @param tbname logical name of the tagboard to flip
     * @exception FTAUnknownObjectException thrown when tagboard does not exist
     * @roseuid 38B3C2E9ED3F
     */
    public void flipOrder(String tbname)
	throws FTAUnknownObjectException {
	_mysyssub.flipOrder(_myensemble.getObjnum(),tbname);
    }
  
    /** 
     * This function is used internally only
     */
    public final boolean runeval() {
	if (!_secondrun) {
	    _secondrun = true;
	    return false;
	}
	return _myensemble.getRunFlag();
    }

    /** 
     * This function is used internally only
     */
    public final boolean belongs(FTAEnsembleSub e) {
	return e == _myensemble;
    }

    /**
     * This is the central routine of an actor, to be implemented by the user.
     * @exception FTAEvalException to be thrown by the user code
     *   if anything occurs that cannot be handled locally.
     * @return the returncode
     *   <dl><dt>FTAActor.KEEPALIVE<dd>return code to reschedule actor
     *   in the next tick again
     *   <dt>FTAActor.TERMINATE<dd>return code to terminate this actor
     *   after this tick.</dl>
     * @roseuid 38B3C2E9ED41
     */
    public int eval() throws FTAEvalException {
	throw new FTAEvalException(_myname + 
				   "actor: Implement eval() method");
    }
  
    /**
     * This constructor is used internally. You cannot use it to create a
     * working actor.
     * @roseuid 38B3C2E9F1DA
     */
    public FTAActor() {
	super();
    }
  
    /**
     * This method is used internally. It is not useful for user applications.
     * @roseuid 38B3C2E9F1DB
     */
    public final void baseclassinit(String name) {
	_myname = name;
	_myensemble = null;
	_secondrun = false;
    }

    /**
     * This method is used internally. It is not useful for user
     * applications.
     */
    public final void baseclassinit(FTAEnsembleSub en) {
	_myensemble = en;
	// obtain a reference to the sysmain
	_mysyssub = _myensemble.getSysSub();
	_secondrun = false;
    }
  
    /**
     * Log a user message to the logging device
     * @param type a user specifyable message class code
     * @param msg the message string to be issued.
     * @roseuid 38B3C2E9F1E1
     * @deprecated Use userMessage(int,String) instead
     */
    public final void userMessage(String type, String msg) {
	userMessage(LOG_STD,type+": "+msg);
    }

    /**
     * Log a user message to the logging device
     * @param type the logging class
     * @param msg the message string to be issued.
     */
    public final void userMessage(int type,String msg) {
	if (_mysyssub != null)
	    _mysyssub.logMessage(type,msg);
	else {
	    System.out.println("userMessage from "+_myname+": "+msg);
	}
    }

    /**
     * ask for the current list of available hosts.
     * Note: this is the list of <b>known</b> hosts, not the
     * list of hosts that participate in this FTASystem.
     * @return a list of hostnames
     */
    public final String[] getHosts() {
	return _mysyssub.getHosts();
    }

    /**
     * get the ensemble time for my own ensemble
     * @param format either return the tick count or the milliseconds
     *   since 1 Jan 1970 of the last time the ensemble ticked
     *   <dl><dt>ABSOLUTE<dd>return milliseconds
     *       <dt>RELATIVE<dd>return tick count</dl>
     * @return the tick time (count or absolute time)
     */
    public final long getEnsembleTime(int format) {
	return _myensemble.getEnsembleTime(format);
    }
  
    /**
     * return the name of my ensemble
     * @return the name of the home ensemble
     */
    public final String getEnsembleName() {
	return _myensemble.getName();
    }

    /**
     * return the name of the host this actor is running on
     */
    public final String getHostName() {
	return _svc.getFQDNname();
    }
  
    /**
     * return the list of tags on a tagboard instance
     * @return the list of tagtypes that are available
     * @param tbname name of the tagboard to check
     * @exception FTAUnknownObjectException thrown when the tagboard is unknown
     */
    public final String[] getTagTypes(String tbname)
	throws FTAUnknownObjectException {
	return _mysyssub.getTagTypes(tbname);
    }

    /**
     * This is the initialization routine of the actor, to be implemented by
     * the user.
     * @param args list of arguments passed by <code>addActor</code> to the
     *   newly created actor. 
     * @exception FTAArgumentException to be thrown by the user code if some
     *   argument is incorrect for this type of actor
     */
    public void init(String[] args) throws FTAArgumentException {
	throw new FTAArgumentException("Init Actor: implement init() method!");
    }

    /**
     * This is the post initialization routine of the actor,
     * to be implementable by the user.
     */
    public void postInit() {
	// do nothing
    }

    /**
     * provide HTML Information about this object and its contained objects
     */
    public FTAServletReply reply(FTAServletRequest req) {
	return FTAServletReply.defaultReply(this,_myname);
    }

    /**
     * return an absolute URL where the object is reachable by default
     * @return url
     */
    public final String getURL(int n) {
	return _myensemble.getURL(1)+"/"+_myname;
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
