// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

//Source file: $Id: FTAInitActor.java,v 1.5 2001/06/21 14:57:09 veit Exp $

package jfta.actors;

import java.rmi.*;

import jfta.exception.*;
import jfta.ensemble.*;
import jfta.base.*;
import jfta.tagboard.*;
import jfta.system.*;
import jfta.tags.*;
import jfta.bootstrap.FTADaemon;

/**
 * Class <code>FTAInitActor</code> is the base class of the init actor
 * that is run once during initialization. This actor is <b>not</b>
 * started in an ensemble context.
 * To create your own init actor, <b>extend</b> your class from
 * <code>FTAInitActor</code>
 * and <b>implement</b> the interface <code>FTAActorI</code>.
 * @author   Holger Veit, Elmar Degen
 * @version  $Id: FTAInitActor.java,v 1.5 2001/06/21 14:57:09 veit Exp $
 * @see      jfta.actors.FTAActor
 * @since    JFTA1.0
 */
public class FTAInitActor extends Object implements FTAActorI {

    /**
     * constant for logMessage
     */
    public final static int LOG_STD = FTADaemon.LOG_STD;
    /**
     * constant for logMessage
     */
    public final static int LOG_ERROR = FTADaemon.LOG_ERROR;
    /**
     * constant for logMessage
     */
    public final static int LOG_WARNING = FTADaemon.LOG_WARNING;
    /**
     * constant for logMessage
     */
    public final static int LOG_SUCCESS = FTADaemon.LOG_SUCCESS;
    /**
     * constant for logMessage
     */
    public final static int LOG_USER = FTADaemon.LOG_USER;

    /**
     * @serial the reference to the current system main office
     */
    protected FTASysSub _mysyssub;
    /**
     * @serial a temporary storage for the internal FTAService helper class
     */
    protected final static FTAService _svc = FTAService.Instance();
  
    /**
     * convert a pathname, forward from FTAService
     * @param path pathname
     * @return converted name
     */
    protected final String expandEnv(String path) {
	return _svc.expandEnv(path);
    }

    /**
     * Returns the name of the actor.
     * @return  the name of the actor as a <code>String</code>
     * @roseuid 38B3C2EC9DE0
     */
    protected final String getName() {
	return "INITACTOR";
    }
  
    /**
     * This constructor is used internally only. It can't be used to create
     * a valid init actor.
     * @roseuid 38B3C2EC9DE1
     */
    protected FTAInitActor() {
	super();
    }
  
    /**
     * Return a hash code for the actor. This is used internally
     * in the management of actors in an ensemble.
     * @return a hash code as an <code>int</code> value.
     * @roseuid 38B3C2EC9DE2
     */
    public int hashCode() {
	return 0xdeadbeef;
    }
  
    /**
     * Register a new host for use by the current FTA system.
     * @param hostname  the fully qualified domain name of a host
     * @exception FTACreationException if the creation failed
     * @roseuid 38B3C2ECDF58
     */
    protected final void addHost(String hostname)
	throws FTACreationException {
	_mysyssub.addHost(hostname);
    }
  
    /**
     * Submit a new tag to a tagboard.
     * @param tbname logical name of the target tagboard.
     * @param tag tag
     * @exception FTAUnknownObjectException the tagboard does not
     *   exist.
     * @exception FTATagException failed to distribute tag
     */
    protected final void putTag(String tbname, FTATag tag)
	throws FTAUnknownObjectException,FTATagException {
	_mysyssub.putTag(tbname,tag);
    }

    /**
     * Submit a new tag to a tagboard.
     * @param tbname logical name of the target tagboard.
     * @param tgtype tag type
     * @param args tag init args
     * @exception FTAUnknownObjectException the tagboard does not
     *   exist.
     * @exception FTATagException the tag could not be distributed.
     * @exception FTAArgumentException user init raised an exception
     */
    protected final void putTag(String tbname, String tgtype,String[] args)
	throws FTAUnknownObjectException,FTATagException,
	FTAArgumentException {
      Object tagobj = null;
      try {
	  Class tagclass = Class.forName(tgtype);
	  tagobj = tagclass.newInstance();
	  FTATag tag = (FTATag)tagobj;
	  tag.init(args);
	  _mysyssub.putTag(tbname,tag);
      }
      catch (FTAArgumentException e) {
	  throw e;
      }
      catch (FTAUnknownObjectException e) {
	  throw e;
      }
      catch (Exception e) {
	  throw new FTATagException("No such tag: "+tgtype);
      }
    }

    /**
     * Establish an ownership relation between an ensemble and a tagboard
     * @param enname logical name of the ensemble that owns a tagboard
     * @param tbname logical name of the tagboard that is owned
     * @exception FTACreationException the ownership relation couldn't have
     *   been created
     * @exception FTAUnknownObjectException the tagboard or ensemble does not
     *   exist, or is non-distributed, but has no office on the host of the
     *   specified partner.
     * @roseuid 38B3C2ECDF69
     */
    protected void addETSync(String enname, String tbname)
	throws FTACreationException,FTAUnknownObjectException {
	_mysyssub.addETSync(enname,tbname);
    }
  
    /**
     * Add a new actor to the an ensemble on a specified host.
     * @param hostname  fully qualified hostname of the ensemble office where
     * the actor is supposed to be created.
     * @param enname  logical name of the ensemble
     * @param acname  logical name of the actor. Must be unique in this FTA
     *   system.
     * @param actype the actor type
     * @param args the actor initialization args
     * @exception FTACreationException  the actor couldn't have been created.
     * @exception FTAUnknownObjectException the specified ensemble does
     * @exception FTAArgumentException user init raised an exception
     * not exist or is not a distributed object that has a local office
     * on the specified host.
     * @roseuid 38B3C2ECDF5F
     */
    protected final void addActor(String hostname, String enname,
				  String acname, FTAActor actor)
	throws FTACreationException,FTAUnknownObjectException,
	FTAArgumentException {
	_mysyssub.addActor(hostname,enname,acname,actor);
    }

    /**
     * Add a new actor to the an ensemble on a specified host.
     * @param hostname  fully qualified hostname of the ensemble office where
     * the actor is supposed to be created.
     * @param acname  logical name of the actor. Must be unique in this FTA
     *   system.
     * @param actor the actor
     * @exception FTACreationException  the actor couldn't have been created.
     * @exception FTAUnknownObjectException the specified ensemble does
     * @exception FTAArgumentException user init raised an exception
     * not exist or is not a distributed object that has a local office
     * on the specified host.
     * @see #init <code>init()</code>
     * @roseuid 38B3C2ECDF5F
     */
    protected final void addActor(String hostname, String enname,
				  String acname, String actype,
				  String[] args)
	throws FTACreationException,FTAUnknownObjectException,
	FTAArgumentException {
	_mysyssub.addActor(hostname,enname,acname,actype,args);
    }
  
    /**
     * Add a new tagboard to the current FTA system.
     * @param tbname  logical name of the tagboard. Must be unique
     * in this FTA system.
     * @param hostname  fully qualified hostname (has to be registered
     *   with {@link #addHost addHost}).
     * @param dist distribution flags
     * @exception FTACreationException  the tagboard could not have been
     *   created.
     * @deprecated @see #addTagboard(String,int)
     */
    protected final void addTagboard(String tbname, String hostname,
				     int flags)
	throws FTACreationException {
	addTagboard(tbname,flags);
    }
  
    /**
     * Add a new tagboard to the current FTA system.
     * @param tbname  logical name of the tagboard. Must be unique
     * in this FTA system.
     * @param flags distribution flags
     * @exception FTACreationException  the tagboard could not have been
     *   created.
     */
    protected final void addTagboard(String tbname, int flags)
	throws FTACreationException {
	_mysyssub.addTagboard(tbname,flags);
    }
  
    /**
     * Add a new ensemble to the current FTA system.
     * @param enname  logical name of the ensemble. Must be unique
     *   in this FTA system.
     * @param hostname  fully qualified hostname (has to be registered
     *   with {@link #addHost addHost}).
     * @param dist distribution flags
     * @exception FTACreationException  the ensemble could not have been
     *   created.
     * @deprecated @see addEnsemble(String,int)
     */
    protected final void addEnsemble(String enname, 
				     String hostname, int flags)
	throws FTACreationException {
	addEnsemble(enname,flags);
    }

    /**
     * Add a new ensemble to the current FTA system.
     * @param enname  logical name of the ensemble. Must be unique
     *   in this FTA system.
     * @param dist distribution flags
     * @exception FTACreationException  the ensemble could not have been
     *   created.
     */
    protected final void addEnsemble(String enname, int flags)
	throws FTACreationException {
	_mysyssub.addEnsemble(enname,flags);
    }

    /**
     * ask for the current list of available hosts.
     * Note: this is the list of <b>known</b> hosts, not the
     * list of hosts that participate in this FTASystem.
     * @return a list of hostnames
     */
    protected final String[] getHosts() {
	return _mysyssub.getHosts();
    }

    /**
     * This is the central routine of an actor, to be implemented by the user.
     * @exception FTAEvalActorException to be thrown by the user code
     * if anything occurs that cannot be handled locally.
     * @return the returncode
     *   <dl><dt>FTAActor.TERMINATE<dd>return code to terminate this
     * actor after this tick.
     * @roseuid 38B3C2ECF10A
     */
    public int eval() throws FTAEvalActorException {
	throw new FTAEvalActorException("Init actor: implement eval() method!");
    }
  
    /**
     * This is the initialization routine of the actor, to be implemented by
     * the user.
     * @param args list of arguments passed by <code>addActor</code> to the
     *   newly created actor. 
     * @exception FTAArgumentException to be thrown by the user code if some
     *   argument is incoreect for this type of actor
     * @roseuid 38B3C2ECF10B
     */
    public void init(String[] args) throws FTAArgumentException {
	throw new FTAArgumentException("Init Actor: implement init() method!");
    }
  
    /**
     * This method is used internally. It is not useful for user
     * applications.
     * @roseuid 38B3C2EDEAE0
     */
    public final void baseclassinit(FTASysSub mysys) {
	_mysyssub = mysys;
    }

    /**
     * Log a user message to the logging device
     * @param type a user specifyable message class code
     * @param msg the message string to be issued.
     * @roseuid 38B3C2EDEAE2
     * @deprecated user userMessage(int,String) instead
     */
    protected final void userMessage(String type, String msg) {
	userMessage(0,type+": "+msg);
    }

    /**
     * Log a user message to the logging device
     * @param type the logging class
     * @param msg the message string to be issued.
     */
    public void userMessage(int type,String msg) {
	_mysyssub.logMessage(type,msg);
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
