// Copyright (c)1999-2001 by "GMD-AiS"
//  Full name: "GMD National Research Center for Information Technology -
//  Institute for Autonomous Intelligent Systems", in future
//  named "Fraunhofer Institute for Autonomous Intelligent Systems"
//
// This software is provided 'as is' by the above copyright holder without
// any explicit or implicit warranty and support.
//
// Within the granted license, the licensee is allowed to freely use
// and modify the software EXCLUSIVELY FOR academical, SCIENTIFIC, and/or
// educational PURPOSES. It is permitted to cite (in the common scientific
// sense) fragments of the code in scientific publications of the licensee,
// and also use modified or unmodified parts of the software in own
// research prototypes.
//
// Commerical use, as well as integration into hardware/software systems or
// other products, other then for aforementioned scientific purposes, is
// PROHIBITED WITHOUT EXPLICIT PERMISSION of the copyright holder.
//
// Prototyping software that makes use of parts of this code have to
// refer to the above copyright when given out to third parties by
// a line
//   "This software contains or uses code that is copyrighted
//    by the GMD National Research Center for Information Technology
//    (Fraunhofer Institute for Autonomous Intelligent Systems)"
// in the user visible part of the software (documentation, about box, etc.)
//
// The copyright holder provides the software 'as is' without any support
// (request for bugfixes, for instance), and without any implicit or explicit
// warranty. In particular, the copyright holder cannot be made liable
// for lack of certain features, for damages of any kind that might result
// from any proper or improper use of this software. This software is not
// to be used in safety-critical environments.
//
// Beyond this license, GMD-AiS appreciates, in the scientific sense,
// reports and code submissions of significant changes or improvements
// of the jFTA core software that might be integrated into a future
// version of this code. Such reports are entirely voluntary, though;
// they are not mandated by this license. Use the address jfta@borneo.gmd.de
// (in future jfta@borneo.ais.fhg.de) for submissions.

//Source file: $Id: FTATag.java,v 1.8 2001/06/21 14:57:27 veit Exp $

package jfta.tags;

import java.io.*;

import jfta.exception.*;
import jfta.base.*;
import jfta.http.*;

/**
 * This is the abstract base class for data tags
 * To create an own tag <b>extend</b> this class and implement
 * the constructor
 * @author Holger Veit, Elmar Degen
 * @version $Id: FTATag.java,v 1.8 2001/06/21 14:57:27 veit Exp $
 * @since JFTA1.0
 */
public abstract class FTATag extends Object 
    implements Serializable,FTAServletI {
    
    /**
     * This is used internally only. Don't touch! Might disappear
     * in future version.
     */
    public transient FTATag link;

    /**
     * This is a default hint for the size of a store to be created
     * for the tags of this type in a tagboard.
     */
    public static int CAPACITY_HINT = 10;
    
    /**
     * tag is not indexed, i.e. will be accessed mainly sequential by
     * tag type. The tag may have an index, though. getTag() using index
     * will result in linear search.
     * An iterator over a subrange of tag indices will result in linear
     * search.
     */
    public static int SEQUENTIAL = 1;

    /**
     * tag has an index, which is sparse, e.g. a String. getTag() using
     * index will typically be a constant time operation.
     * An iterator over a subrange of tag indices will result in linear
     * searching, because there is no ordering.
     */
    public static int SPARSE = 2;

    /**
     * tag is indexed, and the index range is complete (e.g. an integer).
     * getTag() using index will typically be a constant time operation.
     * An iterator over a subrange of tag indices will not cause linear
     * searching but direct access.
     */
    public static int RANDOMACCESS = 3;
    
    /**
     * Return the real tag type.
     * @return the class type of the tag.
     * @roseuid 37F8BE6FAEF0
     */
    public String getType() {
	return getClass().getName();
    }
    
    /**
     * post initialize the tag.
     * This method is called when the tag is received at the
     * target TagboardSub before it is stored. This allows to
     * do late initializations.
     */
    public void postInit() {
	// does nothing by default
    }

    /**
     * Constructor for the tag. This is used internally only, it cannot
     * be used to produce a usable tag.
     * @roseuid 37F8BEC72BA0
     */
    public FTATag() {
	link = null;
    }
    
    /**
     * Initialize the tag, to be implemented by the user.
     * This is used to pass arguments to a newly created tag in order
     * to setup the internal fields.
     * @param args[] The argument vector. The tag init() routine
     * is responsible for parsing the list of arguments correctly
     * and throw an exception in case of an invalid argument.
     * @exception FTAArgumentException will be thrown when one
     * of the arguments is invalid.
     * @roseuid 38B3C56FA2A0
     */
    public void init(String args[]) throws FTAArgumentException {
	throw new FTAArgumentException(getType()+
				       ": init() baseclass called");
    }

    /**
     * provide HTML Information about this object and its contained objects
     */
    public FTAServletReply reply(FTAServletRequest req) {
	return FTAServletReply.defaultReply(this,null);
    }

    /**
     * return an absolute URL where the object is reachable by default
     * @return url
     */
    public String getURL(int n) {
	return null;
    }

    /**
     * return true if the tag implements custom backup/restore routines
     * saveSelf,restoreSelf. You should do this in order to speed up
     * data transfer. By default, the mechanism uses slow default facilities
     */
    public boolean hasSaveRestore() {
	return false;
    }

    /**
     * the custom save method. Although you explicitly need to overwrite
     * hasSaveRestore() method, this method is implemented in this base
     * class, so that it may be used in the following way by a subclass:
     * <code>
     *   public void saveSelf(DataOutputStream out) throws IOException {
     *     super.saveSelf(out);
     *     ...save own attributes...
     *   }
     * </code>
     */
    public void saveSelf(DataOutputStream out) throws IOException {
	// do nothing
    }

    /**
     * the custom restore method. Although you explicitly need to overwrite
     * hasSaveRestore() method, this method is implemented in this base
     * class, so that it may be used in the following way by a subclass:
     * <code>
     *   public void restoreSelf(DataInputStream out) throws IOException {
     *     super.restoreSelf(in);
     *     ...restore own attributes...
     *   }
     * </code>
     */
    public void restoreSelf(DataInputStream out) throws IOException {
	link = null;
    }
}

/*
 * Local variables: 
 * tab-width: 8
 * c-basic-offset: 4
 * End: 
 */
